@@ -1,8 +1,11 @@
 import datetime
 import pendulum
-from singer import utils
+from singer.logger import get_logger
+from singer.utils import strftime
 
 
+LOGGER = get_logger()
+
 NO_INTEGER_DATETIME_PARSING = "no-integer-datetime-parsing"
 UNIX_SECONDS_INTEGER_DATETIME_PARSING = "unix-seconds-integer-datetime-parsing"
 UNIX_MILLISECONDS_INTEGER_DATETIME_PARSING = "unix-milliseconds-integer-datetime-parsing"
@@ -15,15 +18,18 @@ VALID_DATETIME_FORMATS = [
 
 
 def string_to_datetime(value):
-    return utils.strftime(pendulum.parse(value))
+    try:
+        return strftime(pendulum.parse(value))
+    except:
+        return None
 
 
 def unix_milliseconds_to_datetime(value):
-    return utils.strftime(datetime.datetime.utcfromtimestamp(int(value) * 0.001))
+    return strftime(datetime.datetime.utcfromtimestamp(int(value) * 0.001))
 
 
 def unix_seconds_to_datetime(value):
-    return utils.strftime(datetime.datetime.utcfromtimestamp(int(value)))
+    return strftime(datetime.datetime.utcfromtimestamp(int(value)))
 
 
 class SchemaMismatch(Exception):
@@ -32,8 +38,9 @@ class SchemaMismatch(Exception):
             msg = "An error occured during transform that was not a schema mismatch"
 
         else:
-            msg = "Errors during transform\n\t{}".format("\n\t".join(e.tostr() for e in errors))
-            msg += "\nErrors during transform: [{}]".format(", ".join(e.tostr() for e in errors))
+            estrs = [e.tostr() for e in errors]
+            msg = "Errors during transform\n\t{}".format("\n\t".join(estrs))
+            msg += "\n\n\nErrors during transform: [{}]".format(", ".join(estrs))
 
         super(SchemaMismatch, self).__init__(msg)
 
@@ -61,6 +68,15 @@ class Transformer:
         self.removed = set()
         self.errors = []
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        if self.removed:
+            LOGGER.warning("Removed %s paths during transforms:\n\t%s",
+                           len(self.removed),
+                           "\n\t".join(sorted(self.removed)))
+
     def transform(self, data, schema):
         success, transformed_data = self.transform_recur(data, schema, [])
         if not success:
@@ -80,37 +96,25 @@ class Transformer:
             types.remove("null")
             types.append("null")
 
-        types_len = len(types)
-        for i, typ in enumerate(types):
-            try:
-                success, transformed_data = self._transform(data, typ, schema, path)
-                if success:
-                    return success, transformed_data
-                else:
-                    if i == (types_len - 1):
-                        self.errors.append(Error(path, data, schema))
-                        return False, None
-                    else:
-                        pass
-            except:
-                if i == (types_len - 1):
-                    self.errors.append(Error(path, data, schema))
-                    return False, None
-                else:
-                    pass
+        for typ in types:
+            success, transformed_data = self._transform(data, typ, schema, path)
+            if success:
+                return success, transformed_data
+        else: # pylint: disable=useless-else-on-loop
+            # exhaused all types and didn't return, so we failed :-(
+            self.errors.append(Error(path, data, schema))
+            return False, None
 
     def _transform_anyof(self, data, schema, path):
         subschemas = schema['anyOf']
-        subschemas_len = len(subschemas)
-        for i, subschema in enumerate(subschemas):
+        for subschema in subschemas:
             success, transformed_data = self.transform_recur(data, subschema, path)
             if success:
                 return success, transformed_data
-            else:
-                if i == (subschemas_len - 1):
-                    return False, None
-                else:
-                    pass
+        else: # pylint: disable=useless-else-on-loop
+            # exhaused all schemas and didn't return, so we failed :-(
+            self.errors.append(Error(path, data, schema))
+            return False, None
 
     def _transform_object(self, data, schema, path):
         result = {}
@@ -162,7 +166,11 @@ class Transformer:
                 return False, None
 
         elif schema.get("format") == "date-time":
-            return True, self._transform_datetime(data)
+            data = self._transform_datetime(data)
+            if data is None:
+                return False, None
+
+            return True, data
 
         elif typ == "object":
             return self._transform_object(data, schema["properties"], path)
@@ -172,7 +180,10 @@ class Transformer:
 
         elif typ == "string":
             if data != None:
-                return True, str(data)
+                try:
+                    return True, str(data)
+                except:
+                    return False, None
             else:
                 return False, None
 
@@ -180,19 +191,28 @@ class Transformer:
             if isinstance(data, str):
                 data = data.replace(",", "")
 
-            return True, int(data)
+            try:
+                return True, int(data)
+            except:
+                return False, None
 
         elif typ == "number":
             if isinstance(data, str):
                 data = data.replace(",", "")
 
-            return True, float(data)
+            try:
+                return True, float(data)
+            except:
+                return False, None
 
         elif typ == "boolean":
             if isinstance(data, str) and data.lower() == "false":
                 return True, False
 
-            return True, bool(data)
+            try:
+                return True, bool(data)
+            except:
+                return False, None
 
         else:
             return False, None
@@ -219,7 +239,6 @@ def transform(data, schema, integer_datetime_fmt=NO_INTEGER_DATETIME_PARSING, pr
     transformer = Transformer(integer_datetime_fmt, pre_hook)
     return transformer.transform(data, schema)
 
-
 def _transform_datetime(value, integer_datetime_fmt=NO_INTEGER_DATETIME_PARSING):
     transformer = Transformer(integer_datetime_fmt)
     return transformer._transform_datetime(value)
